package hmac

import (
	"crypto/hmac"
	"crypto/sha256"
	"fmt"
	"hash"
	"net/http"

	"gitlab.bad-wolf.fr/wolf/hmac/requestencoder"
)

// ByteEncoder is a function taking a []byte array and returning an encoded
// version of the data. An example may be a function encoding []byte to b64
type ByteEncoder func([]byte) ([]byte, error)

type Signer interface {
	Sign(req *http.Request, key []byte) ([]byte, error)
}

func NewSigner(opts ...SignerOpt) Signer {
	s := &signer{
		hashFunction: sha256.New,
	}
	for _, o := range opts {
		o(s)
	}
	return s
}

type signer struct {
	hashFunction     func() hash.Hash
	requestEncoder   requestencoder.RequestEncoder
	signatureEncoder ByteEncoder
}

func (s *signer) Sign(req *http.Request, key []byte) ([]byte, error) {
	mac := hmac.New(s.hashFunction, key)
	requestString, err := s.requestEncoder(req)
	if err != nil {
		return nil, fmt.Errorf("unable to encode request: %w", err)
	}

	if _, err := mac.Write(requestString); err != nil {
		return nil, err
	}

	signature := mac.Sum(nil)

	if s.signatureEncoder != nil {
		return s.signatureEncoder(signature)
	}
	return signature, nil
}
